import sys, os
import time
import datetime
import urllib.request
import json
import RPi.GPIO as GPIO

import pygame
from pygame.locals import *
from pygame.time import *
import pygame.freetype as freetype

#---------------------------------------------------------------------

# Routine to collect data from Powerwall
def read_energy_values():
    global energy_home
    global energy_solar
    global energy_grid
    global energy_battery
    global energy_charge
    global energy_collected

    #energyURL1 gives power levels
    energyURL1 = "http://" + powerwallIP + "/api/meters/aggregates"

    #energyURL2 gives battery State of Charge
    energyURL2 = "http://" + powerwallIP + "/api/system_status/soe"

    try:
        with urllib.request.urlopen(energyURL1, timeout=1) as response:
            res = response.read().decode('utf-8')
        resData = json.loads(res)

        energy_grid = resData['site']['instant_power']
        energy_solar = resData['solar']['instant_power']
        energy_home = resData['load']['instant_power']
        energy_battery = resData['battery']['instant_power']
        energy_collected = True
    except:
        energy_collected = False

    try:
        with urllib.request.urlopen(energyURL2, timeout=1) as response:
            res = response.read().decode('utf-8')
        resData = json.loads(res)

        energy_charge = resData['percentage']
        energy_collected = True
    except:
        energy_collected = False

#---------------------------------------------------------------------
# Routine to collect weather data
def read_weather_values():
    global weather_temp
    global weather_humidity
    global weather_pressure
    global weather_collected

    with open(weather_data_filename) as file:
        contents = file.read().splitlines()

    try:
        weather_temp = contents[0]
        weather_humidity = contents[1]
        weather_pressure = contents[2]
        weather_collected = True
    except:
        weather_collected = False

#---------------------------------------------------------------------

#Setup variables
#User defined values first

#+++++++++++++++++++++++++++++++++++++++++++++
#  Change the value of powerwallIP to reflect
#  the IP address of your Powerwall gateway
powerwallIP = "192.168.91.1"

#+++++++++++++++++++++++++++++++++++++++++++++
# Specify the location of the weather data on
# the file share system
weather_data_filename = "/mnt/nas/Enviro/weatherdata"

#+++++++++++++++++++++++++++++++++++++++++++++
# Change colorstep to higher values to have 
# the time and date cycle through colours
# quicker. Or reduce the value to have
# slower changes
colorstep = 5

#+++++++++++++++++++++++++++++++++++++++++++++
# Establish "Sleep" parameters for the display
sleep_start_time = "00:00"
sleep_end_time = "07:30"

#+++++++++++++++++++++++++++++++++++++++++++++
# Establish when to automatically dim the display (night use)
dim_start_time = "18:30"

#+++++++++++++++++++++++++++++++++++++++++++++
# Backlight levels for use with the buttons
backlight_top = 100   #for the Top button
backlight_upper = 50  #for the middle top button
backlight_lower = 5   #for the middle bottom button
backlight_bottom = 0  #for the bottom button

# General program variables from here.  Not normally modified
colors = {
    "grey_light"    :   pygame.Color(200, 200, 200),
    "grey_dark"     :   pygame.Color(100, 100, 100),
    "green"         :   pygame.Color(50, 255, 63),
    "red"           :   pygame.Color(220, 30, 30),
    "blue"          :   pygame.Color(0, 255, 255),
    "black"         :   pygame.Color(0,0,0),
    "yellow"        :   pygame.Color(240, 240, 20),
    "orange"        :   pygame.Color(255, 140, 0),
    "white"         :   pygame.Color(255, 255, 255)
}

os.environ["SDL_FBDEV"]="/dev/fb1"

#start with RGB color 0, 255, 0 and then increment Blue first
red = 0
green = 255
blue = 0
redInc = 0
greenInc = 0
blueInc = colorstep

energy_home = 0
energy_solar = 0
energy_grid = 0
energy_battery = 0
energy_charge = 0
energy_collected = False
weather_temp = 0
weather_humidity = 0
weather_pressure = 0
weather_collectec = False

deg_symbol = '\u00b0'

time_x = 5
time_y = 10
time_x_direction = 1
energy_x = 5
energy_y = 80
energy_y_direction = 1
weather_x = 200

#Start the program with backlight at 50%
backlightlevel = backlight_lower

#More display "Sleep" parameters
sleep_timeout_count = 240 #approx 240 counts / minute
sleeptimer = 0

#Convert the Sleep and Dim entries from the user to datetime objects
sleepstart = datetime.datetime.strptime(sleep_start_time + ":00", "%H:%M:%S").time()
sleepend = datetime.datetime.strptime(sleep_end_time + ":00", "%H:%M:%S").time()
dimstart = datetime.datetime.strptime(dim_start_time + ":00", "%H:%M:%S").time()

switchbounce = 0

showdisplay = True

#Setup GPIO system - used for reading push buttons and controlling backlight
GPIO.setwarnings(False)
GPIO.setmode(GPIO.BCM)

#Setup backlight control - controlled via BCM 18 (GPIO 1) with PWM
# use 1000 Hz PWM and start with nominal duty cycle
GPIO.setup(18, GPIO.OUT)
backlight = GPIO.PWM(18,1000)
backlight.start(backlightlevel)

#Setup to read buttons. BCM 17, 22, 23 and 27 (GPIO 0, 3, 4, 2 respectively)
#Buttons are N/O and pull to ground when pressed
GPIO.setup(17, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(22, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(23, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(27, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.add_event_detect(17, GPIO.FALLING)
GPIO.add_event_detect(22, GPIO.FALLING)
GPIO.add_event_detect(23, GPIO.FALLING)
GPIO.add_event_detect(27, GPIO.FALLING)

#Start Pygame
pygame.init()
fontdir = os.path.dirname(os.path.abspath (__file__))
font = freetype.Font(os.path.join (fontdir, "courbd.ttf"))
screen = pygame.display.set_mode((320, 240))
pygame.mouse.set_visible(False)
font.pad = True

#Get the first set of data values
read_energy_values()
read_weather_values()

#Loop forever
while 1:
    #start with blank screen
    screen.fill (colors["black"])

    #get date and time values
    now = datetime.datetime.now()
    sec = int(now.strftime('%S'))

    if sleeptimer > 0:
        sleeptimer -= 1

    #should new energy data be collected?
    ecolor = colors["orange"]
    if (sec == 0) or (sec == 15) or (sec == 30) or (sec == 45):
        if energy_collected == False:
            #Only try once each 15 second period
            get_energy_values = True
            ecolor = colors["white"]
    else:
        energy_collected = False
        get_energy_values = False

    #should new weather data be collected?
    wcolor = colors["orange"]
    if (sec == 7) or (sec == 22) or (sec == 37) or (sec == 52):
        if weather_collected == False:
            #Only try once each 15 second period
            get_weather_values = True
            wcolor = colors["white"]
    else:
        weather_collected = False
        get_weather_values = False

    #Calculate the color of the Time and Date
    # Default has this cycling through 306 gradually changing colours
    red = red + redInc
    green = green + greenInc
    blue = blue + blueInc

    if redInc > 0:
        if red >= 255:
            red = 255
            redInc = 0
            greenInc = 0
            blueInc = -colorstep
    if redInc < 0:
        if red <= 0:
            red = 0
            redInc = 0
            greenInc = 0
            blueInc = colorstep

    if greenInc > 0:
        if green >= 255:
            green = 255
            redInc = -colorstep
            greenInc = 0
            blueInc = 0
    if greenInc < 0:
        if green <= 0:
            green = 0
            redInc = colorstep
            greenInc = 0
            blueInc = 0

    if blueInc > 0:
        if blue >= 255:
            blue = 255
            redInc = 0
            greenInc = -colorstep
            blueInc = 0
    if blueInc < 0:
        if blue <= 0:
            blue = 0
            redInc = 0
            greenInc = colorstep
            blueInc = 0

    tcolor = pygame.Color(red, green, blue)

    #prepare all data values for display
    # ...Time and date
    strtime = now.strftime('%H:%M')
    strdate = now.strftime('%a %d/%m/%Y')

    # ...Energy
    home = '{0:6.1f} kW'.format(float(energy_home)/1000)
    solar = '{0:6.1f} kW'.format(float(energy_solar)/1000)
    grid = '{0:6.1f} kW'.format(float(energy_grid)/1000)
    if grid == '  -0.0 kW':
        grid = '   0.0 kW'
    #Invert the sign of the battery flow. By default the gateway
    # shows -ve values when the battery is charging and +ve when
    # it is supplying power, but I prefer this the other way around
    battery = '{0:6.1f} kW'.format(float(-energy_battery)/1000)
    charge = '{0:6.1f} %'.format(float(energy_charge))

    # ...Weather
    temp = '{0:6.1f}'.format(float(weather_temp))
    tempstr = temp + ' ' + deg_symbol + 'C'
    humiditystr = '{0:6.1f} %'.format(float(weather_humidity))
    pressurestr = '{0:6.1f} hPa'.format(float(weather_pressure))

    #Calculate all text locations. Text moved on screen to prevent
    # burn-in or stuck pixels
    if time_x > 160:
        time_x_direction = -1
    if time_x < 10:
        time_x_direction = 1
    time_x = time_x + time_x_direction

    ty1 = time_y
    ty2 = ty1 + 44

    if energy_y > 125:
        energy_y_direction = -1
    if energy_y < 81:
        energy_y_direction = 1

    energy_y = energy_y + energy_y_direction
    ey1 = energy_y + 25
    ey2 = ey1 + 17
    ey3 = ey2 + 17
    ey4 = ey3 + 17
    ey5 = ey4 + 17

    weather_y = energy_y
    wy1 = weather_y + 25
    wy2 = wy1 + 17
    wy3 = wy2 + 17

    if switchbounce > 0:
        switchbounce -= 1

    #Set button_pressed flag
    button_pressed = False

    #determine if any buttons have been pressed, adjust backlight level
    if GPIO.event_detected(17):
        #Only do this once each second, sort of a software debounce
        if switchbounce == 0:
            #Top most button - backlight full
            backlightlevel = backlight_top
            switchbounce = 4
            button_pressed = True

    if GPIO.event_detected(22):
        #Only do this once each second, sort of a software debounce
        if switchbounce == 0:
            #2nd from top - set backlight to mid level
            backlightlevel = backlight_upper
            switchbounce = 4
            button_pressed = True

    if GPIO.event_detected(23):
        #3rd from top button - set backlight to low level
        if switchbounce == 0:
            backlightlevel = backlight_lower
            switchbounce = 4
            button_pressed = True

    if GPIO.event_detected(27):
        #Bottom button - turn off backlight
        backlightlevel = backlight_bottom
        button_pressed = True

    for event in pygame.event.get():
        if event.type == MOUSEBUTTONDOWN:
            GPIO.cleanup()
            pygame.quit()

    if button_pressed == True:
        sleeptimer = sleep_timeout_count

    #Should the display be dimmed?
    if (now.strftime("%H:%M:%S") == dimstart.strftime("%H:%M:%S")) and (backlightlevel != backlight_lower):
        backlightlevel = backlight_lower

    #Should we sleep?
    if (now.time() >= sleepstart) and (now.time() < sleepend):
        if sleeptimer == 0:
            backlightlevel = backlight_bottom
    elif (now.strftime('%H:%M') == sleepend.strftime('%H:%M')):
        if sleeptimer == 0:
            backlightlevel = backlight_upper

    #If backlight level is zero, don't bother displaying any text
    if backlightlevel == 0:
        showdisplay = False
    else:
        showdisplay = True

    #setup the displayed values
    if showdisplay:
        font.render_to(screen, (time_x, ty1), strtime,
            tcolor, colors["black"], size=48, style=freetype.STYLE_STRONG)
        font.render_to(screen, (time_x, ty2), strdate,
            tcolor, colors["black"], size=18)

        font.render_to(screen, (energy_x, energy_y), "Energy        ",
            ecolor, colors["black"], size=20, style=freetype.STYLE_UNDERLINE)
        font.render_to(screen, (energy_x, ey1), "Home: " + home,
            colors["blue"], colors["black"], size=18)
        font.render_to(screen, (energy_x, ey2), "Solar:" + solar,
            colors["yellow"], colors["black"], size=18)
        font.render_to(screen, (energy_x, ey3), "Grid: " + grid,
            colors["white"], colors["black"], size=18)
        font.render_to(screen, (energy_x, ey4), "Batt: " + battery,
            colors["green"], colors["black"], size=18)
        font.render_to(screen, (energy_x, ey5), "Level:" + charge,
            colors["green"], colors["black"], size=18)

        font.render_to(screen, (weather_x, weather_y), "Weather  ",
            wcolor, colors["black"], size=20, style=freetype.STYLE_UNDERLINE)
        font.render_to(screen, (weather_x,wy1), tempstr,
            colors["green"], colors["black"], size=18)
        font.render_to(screen, (weather_x,wy2), humiditystr,
            colors["green"], colors["black"], size=18)
        font.render_to(screen, (weather_x,wy3), pressurestr,
            colors["green"], colors["black"], size=18)

        if switchbounce > 0:
            font.render_to(screen, (290, 1), str(backlightlevel) + "%",
                colors["grey_dark"], colors["black"], size = 12)

    #Set the required backlight level
    backlight.ChangeDutyCycle(backlightlevel)

    #And update the pygame display to show the new values
    pygame.display.flip()

    #If we need to get new energy values - do it
    if get_energy_values:
        read_energy_values()

    #If we need to get new weather values - do it
    if get_weather_values:
        read_weather_values()

    #Pause the system for 200 milliseconds
    wait(200)

#As the code is written, these lines should never be executed, but here for completeness
pygame.quit()
GPIO.cleanup()
